@@ -32,6 +32,7 @@ struct session_info_st {
 	pthread_mutex_t imgptr_mut;
 	uint16_t lastx;
 	uint16_t lasty;
+	uint32_t lastcnt;
 	int img_color_black;
 	time_t last_used_time;
 	struct session_info_st *_next;
@@ -71,6 +72,7 @@ struct session_info_st *find_session_info(uint32_t sessionid, int createIfNotExi
 			ret->last_used_time = 0;
 			ret->lastx = 65535;
 			ret->lasty = 65535;
+			ret->lastcnt = 0;
 			pthread_mutex_init(&ret->imgptr_mut, NULL);
 
 			ret->_next = session_list;
@@ -151,7 +153,7 @@ struct image_info_st *get_image_str(const char *sessionid_str) {
 	return(get_image(sessionid));
 }
 
-int handle_event(uint32_t sessionid, uint16_t x, uint16_t y, webdraw_event_t type) {
+int handle_event(uint32_t sessionid, uint16_t x, uint16_t y, uint32_t counter, webdraw_event_t type) {
 	struct session_info_st *curr_sess = NULL;
 	FILE *pngfp;
 
@@ -161,6 +163,17 @@ int handle_event(uint32_t sessionid, uint16_t x, uint16_t y, webdraw_event_t typ
 		return(-1);
 	}
 
+	if (type == WEBDRAW_EVENT_MOVE && counter != 0) {
+		if (counter < curr_sess->lastcnt) {
+#ifndef NDEBUG
+			printf("Ignoring out of order move request (sess=%lu, cnt=%lu, lastcnt=%lu)\n", (unsigned long) sessionid, (unsigned long) counter, (unsigned long) curr_sess->lastcnt);
+#endif
+			return(-1);
+		}
+
+		curr_sess->lastcnt = counter;
+	}
+
 	/* Update session with new data */
 	pthread_mutex_lock(&curr_sess->imgptr_mut);
 	if (!curr_sess->imgptr) {
@@ -182,7 +195,7 @@ int handle_event(uint32_t sessionid, uint16_t x, uint16_t y, webdraw_event_t typ
 	}
 
 	if (curr_sess->imgptr && type == WEBDRAW_EVENT_CLICK) {
-		gdImageFilledArc(curr_sess->imgptr, x, y, 5, 5, 0, 360, gdAntiAliased, gdArc);
+		gdImageFilledArc(curr_sess->imgptr, x, y, 6, 6, 0, 360, gdAntiAliased, gdArc);
 	}
 
 	curr_sess->lastx = x;
@@ -196,9 +209,9 @@ int handle_event(uint32_t sessionid, uint16_t x, uint16_t y, webdraw_event_t typ
 }
 
 int handle_event_str(char *str, webdraw_event_t type) {
-	uint32_t sessionid;
+	uint32_t sessionid, counter;
 	uint16_t x, y;
-	char *sessionid_str, *x_str, *y_str;
+	char *sessionid_str, *x_str, *y_str, *counter_str;
 
 	sessionid_str = str;
 	x_str = strchr(sessionid_str, ',');
@@ -215,11 +228,20 @@ int handle_event_str(char *str, webdraw_event_t type) {
 	*y_str = '\0';
 	y_str++;
 
+	counter_str = strchr(y_str, ',');
+	if (counter_str) {
+		*counter_str = '\0';
+		counter_str++;
+		counter = strtoul(counter_str, NULL, 10);
+	} else {
+		counter = 0;
+	}
+
 	sessionid = strtoul(sessionid_str, NULL, 10);
 	x = strtoul(x_str, NULL, 10);
 	y = strtoul(y_str, NULL, 10);
 
-	return(handle_event(sessionid, x, y, type));
+	return(handle_event(sessionid, x, y, counter, type));
 }
 
 /* Not HTTP/1.0 compliant, yet */
@@ -372,7 +394,19 @@ THREAD_FUNCTION_RETURN handle_connection(void *arg) {
 
 		/* Process request */
 #ifndef NDEBUG
-		printf("GET %s\n", request_line_resource);
+		{
+			int gpn_ret;
+			struct sockaddr_in peerinfo;
+			socklen_t peerinfolen = sizeof(peerinfo);
+
+			gpn_ret = getpeername(fd, (struct sockaddr *) &peerinfo, &peerinfolen);
+
+			if (gpn_ret == 0) {
+				printf("[%s] GET %s\n", inet_ntoa(peerinfo.sin_addr), request_line_resource);
+			} else {
+				printf("[??.??.??.??] GET %s\n", request_line_resource);
+			}
+		}
 #endif
 		if (strncmp(request_line_resource, "/event/", 7) == 0) {
 			/* Process an event */
@@ -421,7 +455,7 @@ THREAD_FUNCTION_RETURN handle_connection(void *arg) {
 				http_reply_body_file = NULL;
 				http_reply_content_length = strlen(http_reply_body);
 			}
-		} else if (strcmp(request_line_resource, "/static/page.html") == 0) {
+		} else if (strcmp(request_line_resource, "/static/page.html") == 0 || strcmp(request_line_resource, "/") == 0 || strcmp(request_line_resource, "") == 0) {
 			/* Return a file */
 			http_reply_code = 200;
 			http_reply_msg = "OK";
@@ -676,9 +710,10 @@ int main(int argc, char **argv) {
 		/* Create worker thread for this request */
 		pthcreate_ret = pthread_create(&curr_thread, NULL, handle_connection, currfd_copy);
 		if (pthcreate_ret != 0) {
-			fprintf(stderr, "Error creating thread, closing socket and hoping for the best. pthread_create() returned %i\n", pthcreate_ret);
+			fprintf(stderr, "Error creating thread, closing socket and aborting. pthread_create() returned %i\n", pthcreate_ret);
 			free(currfd_copy);
 			close(currfd);
+			break;
 		}
 	}
 
